open Util
open Parsec
open Settings

let (warning, error) = mk_error "ussm-assemble"

(* Detects the placeholders for modules in the templates *)
let match_module : string -> (string * string * string) option = fun line ->
  if string_match "\\(^.*\\)<\\?module +\\([^ ]+\\) *\\?>\\(.*$\\)" line
  then Some (Str.matched_group 1 line,
	     Str.matched_group 2 line,
	     Str.matched_group 3 line)
  else None

(* like lines_s, with streams instead *)
let str_cons x s = [< 'x; s >]
let rec stream_filter p = parser
  | [< 'x; xs >] -> if p x
                    then [< 'x; stream_filter p xs >]
                    else stream_filter p xs
  | [<        >] -> [<>]
let rec many_str item loc = loc |>
    ((str_cons <$> item <*> many_str item) <|> pure [<>])
let sep_by_str item sep error =
  str_cons <$> item <*> many_str (sep *> (item <?> error))

let lines_str = sep_by_str line nl ""

(* Insert the relevant modules in the template *)
let rec insert_modules : string -> string Stream.t -> string Stream.t =
  let rec aux page s = match match_module s with
    | None                              -> [< 's >]
    | Some (before, module_name, after) ->
      [< aux page before;
         parse_file lines_str ("_ussm/tmp/" ^ page ^ "." ^ module_name);
         aux page after >]
  in fun page -> parser
    | [< 'l; ls >] -> [< aux page l; insert_modules page ls >]
    | [<        >] -> [<>]

let main () = match Array.length Sys.argv with
  | 1 ->
    [<>] |> open_process "ussm-source-files"
         |> run_parser None lines
         |> List.iter (fun src ->
              let target = open_out ("_ussm/www/" ^ remove_ext src ^ ".html") in
              let source =
                if Sys.file_exists ("_ussm/skin." ^ language src ^ ".html")
                then open_in       ("_ussm/skin." ^ language src ^ ".html")
                else open_in        "_ussm/skin.html" in
              source |> Stream.of_channel
                     |> run_parser None lines_str
                     |> insert_modules (remove_ext src)
                     |> Stream.iter (fun line -> output_string target line;
                                                 output_char   target '\n');
              close_out target;
              close_in  source)

  | _ -> error "doesn't take any argument"


let _ = standard_exit main
