(** Extracts a heading from a file (or standard input)

    Usage: ussm-heading key{:subkey} [filename]

    Input : The named file (if provided), or standard input.
    Output: Standard output.

    Return value:
      0 if the whole header is correctly parsed.  If the requirered heading
        is absent, no character is printed on the standard input, and a warning
        is issued on the standar error.

      1 if there is a parse error.  The error will be documented on the
        standard error.

      2 if there is an uncaught exception I did not think about.
        This would be a bug.

    Edge cases:
      ussm-heading prints nothing, and serve no warning in the following cases:
      - The required heading has the form "key : {}"
      - The key path is empty (`ussm-heading ""`, or `ussm-heading :`)
      It does so because it considers you did it on purpose.

      Leading  ':', trailing ':' and several ':' in a row
      are tolerated in the key path
*)

open Util

let(warning, error) = mk_error "ussm-heading"

let main () =
  match Array.length Sys.argv with
    | (2 | 3) as argv_len ->
      let path = Sys.argv.(1) in
      print_string (
        if argv_len = 2
        then try Header.from_channel path stdin
             with Not_found -> error ("key '" ^ path ^ "' not found")
        else try Header.from_file    path Sys.argv.(2)
             with Not_found -> error ("key '" ^ path ^ "' not found" ^
                                      " in file " ^ Sys.argv.(2)));
      flush stdout;
      exit 0

    | _ -> error "Usage: ussm-heading key{:subkey} [filename]"

let _ = standard_exit main
