(** Removes boring filenames, according to the file
    _ussm/skin/_boring

    Input: standard input.  Each line must contain a path.
    Output: standard output.  The same list, without
    the boring filenames

    A path is boring if it contains a boring name.
    A name is boring if it matches any extended regexp of the
    boring parameter from the _ussm/skin/_settings file.
    For instance, those paths are boring:
      _foo
      foo/_bar
      _foo/bar
      foo/_bar/baz

    Bug: rexeges of the form 'foo$' don't work properly.
         Write '.*foo$' instead.
*)

open Util
open Parsec

let (warning,error) = mk_error "ussm-remove-boring"

let is_not_boring : Str.regexp list -> string -> bool =
  fun boring line ->
    line |> Str.split (Str.regexp "/")
         |> List.exists
             (fun name -> List.exists
                            (fun regexp -> Str.string_match regexp name 0)
                            boring)
         |> not

let main () =
  match Array.length Sys.argv with
    | 1 ->
      let boring =
        (try "_ussm/settings"
               |> Header.from_file "boring"
               |> Stream.of_string
               |> run_parser (Some "_ussm/settings") lines
               |> List.map Str.regexp
         with
           | Not_found -> error "Can't find parameter boring") in

      stdin |> Stream.of_channel
            |> run_parser None lines
            |> List.filter (is_not_boring boring)
            |> List.iter print_endline

    | _ -> error "doesn't take any argument."

let _ = standard_exit main
